unit fReceive2;

interface

uses
  Windows, Forms, ScktComp, Grids, StdCtrls, Classes, Controls, Dialogs;

type

{ TStock }
  TStock = class(TPersistent)
  private
    function GetKey: string;
  protected
    r: packed record
      ID: WORD;
      B: Byte;
      W: Word;
      I: Integer;
      S: Single;
      D: Double;
      Data: array[0..4] of Single;
    end;
  public
    constructor Create(const ID: WORD);
    property Key: string read GetKey;
  end;

{ TfrmReceive2 }
  TfrmReceive2 = class(TForm)
    Grid: TStringGrid;
    Button1: TButton;
    procedure Button1Click(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  protected
    FActive: Boolean;
    InTCP: TClientSocket;
    List: TStringList;
    procedure Decode0(S: string);
    procedure Decode1(S: string);
    procedure Decode2(S: string);
    function FindStock(ID: WORD; var Stock: TStock): Boolean;
    procedure IterateData(Stock: TStock);
    procedure DO_ON_READ(Sender: TObject; Socket: TCustomWinSocket);
    procedure DO_ON_ERROR(Sender: TObject; Socket: TCustomWinSocket;
      ErrorEvent: TErrorEvent; var ErrorCode: Integer);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Start();
    procedure Stop();
    procedure Decode(S: string);
  end;

  function EncodeStockKey(ID: WORD): string;

var
  frmReceive2: TfrmReceive2;

implementation

uses SysUtils, fDefine;

{$R *.dfm}

function EncodeStockKey(ID: WORD): string;
begin
  Result := Format('%5.5d', [ID]);
end;

{ TStock }

constructor TStock.Create(const ID: WORD);
begin
  inherited Create;
  r.ID := ID;
end;

function TStock.GetKey: string;
begin
  Result := EncodeStockKey(r.ID);
end;

{ TfrmReceive2 }

constructor TfrmReceive2.Create(AOwner: TComponent);
begin
  inherited;
  List := TStringList.Create;
  List.Duplicates := dupError;
  List.Sorted := True;
  Grid.Cells[0,0] := 'ID';
  Grid.Cells[1,0] := 'B';
  Grid.Cells[2,0] := 'W';
  Grid.Cells[3,0] := 'I';
  Grid.Cells[4,0] := 'S';
  Grid.Cells[5,0] := 'D';
  Grid.Cells[6,0] := 'Data[0]';
  Grid.Cells[7,0] := 'Data[1]';
  Grid.Cells[8,0] := 'Data[2]';
  Grid.Cells[9,0] := 'Data[3]';
  Grid.Cells[10,0]:= 'Data[4]';
end;

destructor TfrmReceive2.Destroy;
var
  I: Integer;
begin
  for I := 0 to List.Count-1 do
    List.Objects[I].Free;
  FreeAndNil(List);
  inherited;
end;

procedure TfrmReceive2.Decode(S: string);
begin
  case Ord(S[5]) of
    0: Decode0(S);
    1: Decode1(S);
    2: Decode2(S);
  end;
end;

function TfrmReceive2.FindStock(ID: WORD; var Stock: TStock): Boolean;
var
  Key: string;
  Index: Integer;
begin
  Key := EncodeStockKey(ID);
  if not List.Find(Key, Index) then
    Index := List.AddObject(Key, TStock.Create(ID));
  if Index <> -1 then
    Stock := TStock(List.Objects[Index])
  else Stock := nil;
  Result := Stock <> nil;
end;

procedure TfrmReceive2.Decode0(S: string);
var
  P: PPacket;
  Stock: TStock;
begin
  P := PPacket(S);
  if FindStock(P.r.p0.ID, Stock) then
  begin
    with Stock, P.r.p0 do
    begin
      r.B := B;
      r.W := W;
      r.I := I;
      r.S := S;
      r.D := D;
    end;
    IterateData(Stock);
  end;
end;

procedure TfrmReceive2.Decode1(S: string);
var
  P: PPacket;
  Stock: TStock;
begin
  P := PPacket(S);
  if FindStock(P.r.p1.ID, Stock) then
  begin
    with Stock, P.r.p1 do
    begin
      r.I := I;
      r.S := S;
      r.D := D;
    end;
    IterateData(Stock);
  end;
end;

procedure TfrmReceive2.Decode2(S: string);
var
  J: Integer;
  P: PPacket;
  Stock: TStock;
begin
  P := PPacket(S);
  if FindStock(P.r.p2.ID, Stock) then
  begin
    with Stock, P.r.p2 do
      for J := 0 to Count-1 do
        r.Data[J] := Data[J];
    IterateData(Stock);
  end;
end;

procedure TfrmReceive2.IterateData(Stock: TStock);
var
  C, R: Integer;
begin
  R := Stock.r.ID mod 10 + 1;
  with Grid do
    for C := 0 to ColCount-1 do
      case C of
        0: Cells[C, R] := Stock.Key;
        1: Cells[C, R] := Format('%d', [Stock.r.B]);
        2: Cells[C, R] := Format('%d', [Stock.r.W]);
        3: Cells[C, R] := Format('%d', [Stock.r.I]);
        4: Cells[C, R] := Format('%.2f', [Stock.r.S]);
        5: Cells[C, R] := Format('%.2f', [Stock.r.D]);
        6..10: Cells[C, R] := Format('%.2f', [Stock.r.Data[C-6]]);
      end;
end;

procedure TfrmReceive2.Start;
begin
  FActive := True;
  Button1.Caption := 'Stop';
  if InTCP = nil then
  begin
    InTCP := TClientSocket.Create(Self);
    InTCP.Address := '127.0.0.1';
    InTCP.Port := 54321;
    InTCP.OnRead := DO_ON_READ;
    InTCP.OnError := DO_ON_ERROR;
    InTCP.Active := True;
  end;
end;

procedure TfrmReceive2.Stop;
begin
  FActive := False;
  Button1.Caption := 'Start';
  if InTCP <> nil then
  begin
    InTCP.Active := False;
    InTCP.OnRead := nil;
    InTCP.OnError := nil;
    InTCP.Address := '';
    InTCP.Port := 0;
    FreeAndNil(InTCP);
  end;
end;

procedure TfrmReceive2.Button1Click(Sender: TObject);
begin
  FActive := not FActive;
  case FActive of
    True: Start();
    else  Stop();
  end;
end;

procedure TfrmReceive2.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := not FActive;
  if not CanClose then
    ShowMessage('Stop First !!!');

end;

procedure TfrmReceive2.DO_ON_READ(Sender: TObject; Socket: TCustomWinSocket);
var
  S: string;
begin
  S := Socket.ReceiveText;
  _decode_(PChar(S), Length(S), Decode);
end;

procedure TfrmReceive2.DO_ON_ERROR(Sender: TObject;
  Socket: TCustomWinSocket; ErrorEvent: TErrorEvent;
  var ErrorCode: Integer);
begin
  Stop();
end;

end.
